/* ztclDecls.h --
   
   Part of: ZTCL
   Contents: declarations for the stubs
   Date: Sat Jul  5, 2003
   
   Abstract
   
	This file  collects all the declarations common  to the internal
	and external interfaces.
   
   Copyright (c) 2003, 2004 Marco Maggi
   
   The author hereby grant  permission to use, copy, modify, distribute,
   and  license this  software and  its documentation  for  any purpose,
   provided that  existing copyright notices are retained  in all copies
   and that  this notice is  included verbatim in any  distributions. No
   written agreement, license, or royalty fee is required for any of the
   authorized uses.   Modifications to this software  may be copyrighted
   by their  authors and need  not follow the licensing  terms described
   here, provided that the new  terms are clearly indicated on the first
   page of each file where they apply.
   
   IN NO EVENT  SHALL THE AUTHOR OR DISTRIBUTORS BE  LIABLE TO ANY PARTY
   FOR DIRECT,  INDIRECT, SPECIAL, INCIDENTAL,  OR CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE USE OF  THIS SOFTWARE, ITS  DOCUMENTATION, OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR HAVE  BEEN ADVISED  OF THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR AND  DISTRIBUTORS SPECIFICALLY  DISCLAIM  ANY WARRANTIES,
   INCLUDING,   BUT  NOT   LIMITED   TO,  THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A   PARTICULAR    PURPOSE,   AND
   NON-INFRINGEMENT.  THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, AND
   THE   AUTHOR  AND   DISTRIBUTORS  HAVE   NO  OBLIGATION   TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
   
*/

#ifndef __ZTCLDECLS_H
#define __ZTCLDECLS_H 1


/** ------------------------------------------------------------
 ** Preprocessor symbols.
 ** ----------------------------------------------------------*/

#define ZTCL_LEVEL_DEFAULT	6
#define ZTCL_LEVEL_FAST		1
#define ZTCL_LEVEL_BEST		9

#define ZTCL_STRATEGY_DEFAULT	(&(Ztcl_Strategies[0]))
#define ZTCL_STRATEGY_FILTERED	(&(Ztcl_Strategies[1]))
#define ZTCL_STRATEGY_HUFFMAN	(&(Ztcl_Strategies[2]))
#define ZTCL_STRATEGY_RLE	(&(Ztcl_Strategies[3]))


/** ------------------------------------------------------------
 ** Types declarations.
 ** ----------------------------------------------------------*/

struct Ztcl_StrategyTag {
  CONST char *	name;
  int		value;
};
typedef CONST struct Ztcl_StrategyTag *		Ztcl_Strategy;

EXTERN CONST struct Ztcl_StrategyTag		Ztcl_Strategies[];

typedef struct Ztcl_Config {
  int			level;
  Ztcl_Strategy		strategy;
} Ztcl_Config;

#define ZTCL_DEFAULT_CONFIG	{ ZTCL_LEVEL_DEFAULT, ZTCL_STRATEGY_DEFAULT }

typedef ClientData		Ztcl_File;
typedef ClientData		Ztcl_Stream;


/** ------------------------------------------------------------
 ** Compression and uncompression function prototypes.
 ** ----------------------------------------------------------*/

#ifndef USE_ZTCL_STUB

EXTERN More_Error Ztcl_CompressObj	_ANSI_ARGS_((Tcl_Obj *srcObj,
						     Ztcl_Config *config,
						     Tcl_Obj **dstObjVar));

EXTERN More_Error Ztcl_DecompressObj	_ANSI_ARGS_((Tcl_Obj *srcObj,
						     Ztcl_Config *config,
						     int size,
						     Tcl_Obj **dstObjVar));

#endif /* not defined USE_ZTCL_STUB */


/** ------------------------------------------------------------
 ** GZ file function prototypes.
 ** ----------------------------------------------------------*/

#ifndef USE_ZTCL_STUB

EXTERN More_Error Ztcl_Open		_ANSI_ARGS_((CONST char *fileName,
						     int openMode,
						     Ztcl_File *tokenVar));
EXTERN More_Error Ztcl_Close		_ANSI_ARGS_((Ztcl_File file));
EXTERN More_Error Ztcl_Read		_ANSI_ARGS_((Ztcl_File file,
						     More_Block *blockVar));
EXTERN More_Error Ztcl_ReadObj		_ANSI_ARGS_((Ztcl_File token,
						     int number,
						     Tcl_Obj **dstObjVar));
EXTERN More_Error Ztcl_Write		_ANSI_ARGS_((Ztcl_File file,
						     More_Block block));
EXTERN More_Error Ztcl_WriteObj		_ANSI_ARGS_((Ztcl_File file,
						     Tcl_Obj *srcObj));
EXTERN int        Ztcl_Eof		_ANSI_ARGS_((Ztcl_File file));
EXTERN More_Error Ztcl_Flush		_ANSI_ARGS_((Ztcl_File file));
EXTERN More_Error Ztcl_Seek		_ANSI_ARGS_((Ztcl_File file,
						     int offset, int whence));
EXTERN More_Error Ztcl_Tell		_ANSI_ARGS_((Ztcl_File file,
						     int *positionVar));

EXTERN More_Error Ztcl_SetLevel		_ANSI_ARGS_((Ztcl_File file,
						     int level));
EXTERN int        Ztcl_GetLevel		_ANSI_ARGS_((Ztcl_File file));
EXTERN More_Error Ztcl_SetStrategy	_ANSI_ARGS_((Ztcl_File file,
						     Ztcl_Strategy strategy));
EXTERN Ztcl_Strategy Ztcl_GetStrategy	_ANSI_ARGS_((Ztcl_File file));
EXTERN int        Ztcl_ReadableFile	_ANSI_ARGS_((Ztcl_File file));
EXTERN int        Ztcl_WritableFile	_ANSI_ARGS_((Ztcl_File file));

#endif /* not defined USE_ZTCL_STUB */


/** ------------------------------------------------------------
 ** Stream functions.
 ** ----------------------------------------------------------*/

#ifndef USE_ZTCL_STUB

EXTERN More_Error Ztcl_StreamInit	_ANSI_ARGS_((Ztcl_Stream *tokenVar,
						     int compress,
						     Ztcl_Config *config));
EXTERN void Ztcl_StreamFinal		_ANSI_ARGS_((Ztcl_Stream token));
EXTERN More_Block Ztcl_StreamOutput	_ANSI_ARGS_((Ztcl_Stream token));
EXTERN void Ztcl_StreamRead		_ANSI_ARGS_((Ztcl_Stream token,
						     int numberOfReadBytes));
EXTERN More_Error Ztcl_StreamWrite	_ANSI_ARGS_((Ztcl_Stream token,
						     More_Block *inputPtr));
EXTERN More_Error Ztcl_StreamFinish	_ANSI_ARGS_((Ztcl_Stream token));
EXTERN More_Error Ztcl_StreamFlush	_ANSI_ARGS_((Ztcl_Stream token));
EXTERN void Ztcl_StreamSetBufferSize	_ANSI_ARGS_((Ztcl_Stream token,
						     int size));
EXTERN void Ztcl_StreamSetBufferHysteresis _ANSI_ARGS_((Ztcl_Stream token,
							int hysteresis));
EXTERN int Ztcl_StreamGetBufferSize	_ANSI_ARGS_((Ztcl_Stream token));
EXTERN int Ztcl_StreamGetBufferHysteresis _ANSI_ARGS_((Ztcl_Stream token));
EXTERN int Ztcl_StreamBufferAllocated	_ANSI_ARGS_((Ztcl_Stream token));
EXTERN int Ztcl_StreamGetLevel		_ANSI_ARGS_((Ztcl_Stream token));
EXTERN int Ztcl_StreamFinished		_ANSI_ARGS_((Ztcl_Stream token));

#endif /* not defined USE_ZTCL_STUB */


/** ------------------------------------------------------------
 ** Channel interface function prototypes.
 ** ----------------------------------------------------------*/

#ifndef USE_ZTCL_STUB

EXTERN Tcl_Channel Ztcl_MakeChannel	_ANSI_ARGS_((Ztcl_File file));

EXTERN Tcl_Channel Ztcl_MakeTransform	_ANSI_ARGS_((Ztcl_Stream input,
						     Ztcl_Stream output,
						     Tcl_Channel channel));

#endif /* not defined USE_ZTCL_STUB */


/** ------------------------------------------------------------
 ** Object function prototypes.
 ** ----------------------------------------------------------*/

#ifndef USE_ZTCL_STUB

EXTERN int	Ztcl_GetLevelFromObj	_ANSI_ARGS_((Tcl_Interp * interp,
						     Tcl_Obj * obj,
						     int *levelVar));

EXTERN int	Ztcl_GetStrategyFromObj	_ANSI_ARGS_((Tcl_Interp * interp,
						     Tcl_Obj * objPtr,
						     Ztcl_Strategy *stratVar));

EXTERN int	Ztcl_IsLevel		_ANSI_ARGS_((int level));
EXTERN int	Ztcl_IsStrategy		_ANSI_ARGS_((Ztcl_Strategy s));

#endif /* not defined USE_ZTCL_STUB */


/** ------------------------------------------------------------
 ** Miscellaneous function prototypes.
 ** ----------------------------------------------------------*/

#ifndef USE_ZTCL_STUB

EXTERN CONST char * Ztcl_Version	_ANSI_ARGS_((void));

EXTERN More_Error Ztcl_MakeErrorDescriptor	_ANSI_ARGS_((int zerror));

#endif /* not defined USE_ZTCL_STUB */


/** ------------------------------------------------------------
 ** Stub table.
 ** ----------------------------------------------------------*/

typedef struct Ztcl_StubTable {

  /* Compress and decompress functions. */

  More_Error (*ztcl_CompressObj)	_ANSI_ARGS_((Tcl_Obj *srcObj,
						     Ztcl_Config *config,
						     Tcl_Obj **dstObjVar));
  
  More_Error (*ztcl_DecompressObj)	_ANSI_ARGS_((Tcl_Obj *srcObj,
						     Ztcl_Config *config,
						     int size,
						     Tcl_Obj **dstObjVar));
  
  /* GZ file functions. */

  More_Error	(*ztcl_Open)		_ANSI_ARGS_((CONST char *fileName,
						     int openMode,
						     Ztcl_File *tokenVar));
  More_Error	(*ztcl_Close)		_ANSI_ARGS_((Ztcl_File file));
  More_Error	(*ztcl_Read)		_ANSI_ARGS_((Ztcl_File file,
						     More_Block *blockVar));
  More_Error	(*ztcl_ReadObj)		_ANSI_ARGS_((Ztcl_File token,
						     int number,
						     Tcl_Obj **dstObjVar));
  More_Error	(*ztcl_Write)		_ANSI_ARGS_((Ztcl_File file,
						     More_Block block));
  More_Error	(*ztcl_WriteObj)	_ANSI_ARGS_((Ztcl_File file,
						     Tcl_Obj *srcObj));
  int		(*ztcl_Eof)		_ANSI_ARGS_((Ztcl_File file));
  More_Error	(*ztcl_Flush)		_ANSI_ARGS_((Ztcl_File file));
  More_Error	(*Ztcl_Seek)		_ANSI_ARGS_((Ztcl_File file,
						     int offset, int whence));
  More_Error	(*ztcl_Tell)		_ANSI_ARGS_((Ztcl_File file,
						     int *positionVar));

  More_Error	(*ztcl_SetLevel)	_ANSI_ARGS_((Ztcl_File file,
						     int level));
  int		(*ztcl_GetLevel)	_ANSI_ARGS_((Ztcl_File file));
  More_Error	(*ztcl_SetStrategy)	_ANSI_ARGS_((Ztcl_File file,
						     Ztcl_Strategy strategy));
  Ztcl_Strategy (*ztcl_GetStrategy)	_ANSI_ARGS_((Ztcl_File file));
  int	        (*ztcl_ReadableFile)	_ANSI_ARGS_((Ztcl_File file));
  int		(*ztcl_WritableFile)	_ANSI_ARGS_((Ztcl_File file));

  
  /* Stream functions. */

  More_Error	(*ztcl_StreamInit)	_ANSI_ARGS_((Ztcl_Stream *tokenVar,
						     int compress,
						     Ztcl_Config *config));
  void		(*ztcl_StreamFinal)	_ANSI_ARGS_((Ztcl_Stream token));
  More_Block	(*ztcl_StreamOutput)	_ANSI_ARGS_((Ztcl_Stream token));
  void		(*ztcl_StreamRead)	_ANSI_ARGS_((Ztcl_Stream token,
						     int numberOfReadBytes));
  More_Error	(*ztcl_StreamWrite)	_ANSI_ARGS_((Ztcl_Stream token,
						     More_Block *inputPtr));
  More_Error	(*ztcl_StreamFinish)	_ANSI_ARGS_((Ztcl_Stream token));
  More_Error	(*ztcl_StreamFlush)	_ANSI_ARGS_((Ztcl_Stream token));
  void		(*ztcl_StreamSetBufferSize) _ANSI_ARGS_((Ztcl_Stream token,
							 int size));
  void		(*ztcl_StreamSetBufferHysteresis) _ANSI_ARGS_((Ztcl_Stream token,
							       int hysteresis));
  int		(*ztcl_StreamGetBufferSize) _ANSI_ARGS_((Ztcl_Stream token));
  int		(*ztcl_StreamGetBufferHysteresis) _ANSI_ARGS_((Ztcl_Stream token));
  int		(*ztcl_StreamBufferAllocated) _ANSI_ARGS_((Ztcl_Stream token));
  int		(*ztcl_StreamGetLevel)	_ANSI_ARGS_((Ztcl_Stream token));
  int		(*ztcl_StreamFinished)	_ANSI_ARGS_((Ztcl_Stream token));

  /* Channel interface functions. */

  Tcl_Channel	(*ztcl_MakeChannel)	_ANSI_ARGS_((Ztcl_File file));

  Tcl_Channel	(*ztcl_MakeTransform)	_ANSI_ARGS_((Ztcl_Stream input,
						     Ztcl_Stream output,
						     Tcl_Channel channel));
  
  /* Object functions. */

  int		(*ztcl_GetLevelFromObj)	_ANSI_ARGS_((Tcl_Interp * interp,
						     Tcl_Obj * obj,
						     int *levelVar));
  
  int		(*ztcl_GetStrategyFromObj) _ANSI_ARGS_((Tcl_Interp * interp,
						     Tcl_Obj * objPtr,
						     Ztcl_Strategy *stratVar));

  int		(*ztcl_IsLevel)		_ANSI_ARGS_((int level));
  int		(*ztcl_IsStrategy)	_ANSI_ARGS_((Ztcl_Strategy s));
  
  /* Miscellaneous functions. */

  CONST char * (*ztcl_Version)		_ANSI_ARGS_((void));
  
} Ztcl_StubTable;


/** ------------------------------------------------------------
 ** Stub declarations.
 ** ----------------------------------------------------------*/

#ifdef USE_ZTCL_STUB

EXTERN CONST Ztcl_StubTable *	ztclStubPtr;

EXTERN CONST char *	Ztcl_InitStub	_ANSI_ARGS_((Tcl_Interp *interp,
						     char *version,
						     int exact));

/* Compress functions. */

#define Ztcl_CompressObj	(ztclStubPtr->ztcl_CompressObj)
#define Ztcl_DecompressObj	(ztclStubPtr->ztcl_DecompressObj)
  
/* File functions. */

#define Ztcl_Open		(ztclStubPtr->ztcl_Open)
#define Ztcl_Close		(ztclStubPtr->ztcl_Close)
#define Ztcl_Read		(ztclStubPtr->ztcl_Read)
#define Ztcl_ReadObj		(ztclStubPtr->ztcl_ReadObj)
#define Ztcl_Write		(ztclStubPtr->ztcl_Write)
#define Ztcl_WriteObj		(ztclStubPtr->ztcl_WriteObj)
#define Ztcl_Eof		(ztclStubPtr->ztcl_Eof)
#define Ztcl_Flush		(ztclStubPtr->ztcl_Flush)
#define Ztcl_Seek		(ztclStubPtr->ztcl_Seek)
#define Ztcl_Tell		(ztclStubPtr->ztcl_Tell)
#define Ztcl_SetLevel		(ztclStubPtr->ztcl_SetLevel)
#define Ztcl_GetLevel		(ztclStubPtr->ztcl_GetLevel)
#define Ztcl_SetStrategy	(ztclStubPtr->ztcl_SetStrategy)
#define Ztcl_GetStrategy	(ztclStubPtr->ztcl_GetStrategy)
#define Ztcl_ReadableFile	(ztclStubPtr->ztcl_ReadableFile)
#define Ztcl_WritableFile	(ztclStubPtr->ztcl_WritableFile)

/* Stream functions. */

#define Ztcl_StreamInit		(ztclStubPtr->ztcl_StreamInit)
#define Ztcl_StreamFinal	(ztclStubPtr->ztcl_StreamFinal)
#define Ztcl_StreamOutput	(ztclStubPtr->ztcl_StreamOutput)
#define Ztcl_StreamRead		(ztclStubPtr->ztcl_StreamRead)
#define Ztcl_StreamWrite	(ztclStubPtr->ztcl_StreamWrite)
#define Ztcl_StreamFinish	(ztclStubPtr->ztcl_StreamFinish)
#define Ztcl_StreamFlush	(ztclStubPtr->ztcl_StreamFlush)
#define Ztcl_StreamSetBufferSize	(ztclStubPtr->ztcl_StreamSetBufferSize)
#define Ztcl_StreamSetBufferHysteresis	(ztclStubPtr->ztcl_StreamSetBufferHysteresis)
#define Ztcl_StreamGetBufferSize	(ztclStubPtr->ztcl_StreamGetBufferSize)
#define Ztcl_StreamGetBufferHysteresis	(ztclStubPtr->ztcl_StreamGetBufferHysteresis)
#define Ztcl_StreamBufferAllocated	(ztclStubPtr->ztcl_StreamBufferAllocated)
#define Ztcl_StreamGetLevel	(ztclStubPtr->ztcl_StreamGetLevel)
#define Ztcl_StreamFinished	(ztclStubPtr->ztcl_StreamFinished)

/* Channel functions. */

#define Ztcl_MakeChannel	(ztclStubPtr->ztcl_MakeChannel)
#define Ztcl_MakeTransform	(ztclStubPtr->ztcl_MakeTransform)

/* Object Functions. */

#define Ztcl_GetLevelFromObj	(ztclStubPtr->ztcl_GetLevelFromObj)
#define Ztcl_GetStrategyFromObj	(ztclStubPtr->ztcl_GetStrategyFromObj)
#define Ztcl_IsLevel		(ztclStubPtr->ztcl_IsLevel)
#define Ztcl_IsStrategy		(ztclStubPtr->ztcl_IsStrategy)
  
/* Miscellaneous functions. */

#define Ztcl_Version		(ztclStubPtr->ztcl_Version)

#endif /* defined USE_ZTCL_STUB */
#endif /* __ZTCLDECLS_H */

/* end of file */
/*
  Local Variables:
  page-delimiter: "^$"
  End:
*/
